@@ -25,7 +25,7 @@
 #include <config.h>
 #include "nautilus-bookmark.h"
 
-#include "nautilus-icon-factory.h"
+#include "nautilus-file.h"
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -35,11 +35,11 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkiconfactory.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
+#include <libnautilus-private/nautilus-file.h>
 
 enum {
 	APPEARANCE_CHANGED,
@@ -50,14 +50,19 @@ enum {
 #define GENERIC_BOOKMARK_ICON_NAME	"gnome-fs-bookmark"
 #define MISSING_BOOKMARK_ICON_NAME	"gnome-fs-bookmark-missing"
 
+#define ELLIPSISED_MENU_ITEM_MIN_CHARS  32
+
 static guint signals[LAST_SIGNAL];
 
 struct NautilusBookmarkDetails
 {
 	char *name;
-	char *uri;
-	char *icon;
+	gboolean has_custom_name;
+	GFile *location;
+	GIcon *icon;
 	NautilusFile *file;
+	
+	char *scroll_file;
 };
 
 static void	  nautilus_bookmark_connect_file	  (NautilusBookmark	 *file);
@@ -80,8 +85,11 @@ nautilus_bookmark_finalize (GObject *object)
 	nautilus_bookmark_disconnect_file (bookmark);	
 
 	g_free (bookmark->details->name);
-	g_free (bookmark->details->uri);
-	g_free (bookmark->details->icon);
+	g_object_unref (bookmark->details->location);
+	if (bookmark->details->icon) {
+		g_object_unref (bookmark->details->icon);
+	}
+	g_free (bookmark->details->scroll_file);
 	g_free (bookmark->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -142,13 +150,13 @@ nautilus_bookmark_compare_with (gconstpointer a, gconstpointer b)
 	bookmark_a = NAUTILUS_BOOKMARK (a);
 	bookmark_b = NAUTILUS_BOOKMARK (b);
 
-	if (strcmp (bookmark_a->details->name,
-		    bookmark_b->details->name) != 0) {
+	if (eel_strcmp (bookmark_a->details->name,
+			bookmark_b->details->name) != 0) {
 		return 1;
 	}
 
-	if (!eel_uris_match (bookmark_a->details->uri,
-			     bookmark_b->details->uri)) {
+	if (!g_file_equal (bookmark_a->details->location,
+			   bookmark_b->details->location)) {
 		return 1;
 	}
 	
@@ -177,8 +185,8 @@ nautilus_bookmark_compare_uris (gconstpointer a, gconstpointer b)
 	bookmark_a = NAUTILUS_BOOKMARK (a);
 	bookmark_b = NAUTILUS_BOOKMARK (b);
 
-	return !eel_uris_match (bookmark_a->details->uri,
-				bookmark_b->details->uri);
+	return !g_file_equal (bookmark_a->details->location,
+			      bookmark_b->details->location);
 }
 
 NautilusBookmark *
@@ -187,8 +195,9 @@ nautilus_bookmark_copy (NautilusBookmark *bookmark)
 	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (bookmark), NULL);
 
 	return nautilus_bookmark_new_with_icon (
-			bookmark->details->uri,
+			bookmark->details->location,
 			bookmark->details->name,
+			bookmark->details->has_custom_name,
 			bookmark->details->icon);
 }
 
@@ -200,13 +209,25 @@ nautilus_bookmark_get_name (NautilusBookmark *bookmark)
 	return g_strdup (bookmark->details->name);
 }
 
+
+gboolean
+nautilus_bookmark_get_has_custom_name (NautilusBookmark *bookmark)
+{
+	g_return_val_if_fail(NAUTILUS_IS_BOOKMARK (bookmark), FALSE);
+
+	return (bookmark->details->has_custom_name);
+}
+
+
 GdkPixbuf *	    
 nautilus_bookmark_get_pixbuf (NautilusBookmark *bookmark,
-			      guint icon_size,
-			      gboolean optimize_for_anti_aliasing)
+			      GtkIconSize stock_size)
 {
 	GdkPixbuf *result;
-	char *icon;
+	GIcon *icon;
+	NautilusIconInfo *info;
+	int pixel_size;
+
 	
 	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (bookmark), NULL);
 
@@ -215,16 +236,17 @@ nautilus_bookmark_get_pixbuf (NautilusBookmark *bookmark,
 		return NULL;
 	}
 	
-	result = nautilus_icon_factory_get_pixbuf_for_icon
-		(icon, NULL, NULL, 
-		 icon_size, NULL, TRUE);
-	
-	g_free (icon);
+	pixel_size = nautilus_get_icon_size_for_stock_size (stock_size);
+	info = nautilus_icon_info_lookup (icon, pixel_size);
+	result = nautilus_icon_info_get_pixbuf_at_size (info, pixel_size);	
+	g_object_unref (info);
+
+	g_object_unref (icon);
 	
 	return result;
 }
 
-char *
+GIcon *
 nautilus_bookmark_get_icon (NautilusBookmark *bookmark)
 {
 	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (bookmark), NULL);
@@ -232,11 +254,14 @@ nautilus_bookmark_get_icon (NautilusBookmark *bookmark)
 	/* Try to connect a file in case file exists now but didn't earlier. */
 	nautilus_bookmark_connect_file (bookmark);
 
-	return g_strdup (bookmark->details->icon);
+	if (bookmark->details->icon) {
+		return g_object_ref (bookmark->details->icon);
+	}
+	return NULL;
 }
 
-char *
-nautilus_bookmark_get_uri (NautilusBookmark *bookmark)
+GFile *
+nautilus_bookmark_get_location (NautilusBookmark *bookmark)
 {
 	g_return_val_if_fail(NAUTILUS_IS_BOOKMARK (bookmark), NULL);
 
@@ -248,7 +273,19 @@ nautilus_bookmark_get_uri (NautilusBookmark *bookmark)
 	 */
 	nautilus_bookmark_connect_file (bookmark);
 
-	return g_strdup (bookmark->details->uri);
+	return g_object_ref (bookmark->details->location);
+}
+
+char *
+nautilus_bookmark_get_uri (NautilusBookmark *bookmark)
+{
+	GFile *file;
+	char *uri;
+
+	file = nautilus_bookmark_get_location (bookmark);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
+	return uri;
 }
 
 
@@ -278,14 +315,24 @@ nautilus_bookmark_set_name (NautilusBookmark *bookmark, const char *new_name)
 	return TRUE;
 }
 
+void
+nautilus_bookmark_set_has_custom_name (NautilusBookmark *bookmark, gboolean has_custom_name)
+{
+	bookmark->details->has_custom_name = has_custom_name;
+}
+
 static gboolean
 nautilus_bookmark_icon_is_different (NautilusBookmark *bookmark,
-		   		     char *new_icon)
+		   		     GIcon *new_icon)
 {
 	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
 	g_assert (new_icon != NULL);
 
-	return eel_strcmp (bookmark->details->icon, new_icon) != 0;
+	if (bookmark->details->icon == NULL) {
+		return new_icon == NULL;
+	}
+	
+	return !g_icon_equal (bookmark->details->icon, new_icon) != 0;
 }
 
 /**
@@ -295,7 +342,7 @@ nautilus_bookmark_icon_is_different (NautilusBookmark *bookmark,
 static gboolean
 nautilus_bookmark_update_icon (NautilusBookmark *bookmark)
 {
-	char *new_icon;
+	GIcon *new_icon;
 
 	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
 
@@ -303,14 +350,17 @@ nautilus_bookmark_update_icon (NautilusBookmark *bookmark)
 		return FALSE;
 	}
 
-	if (nautilus_icon_factory_is_icon_ready_for_file (bookmark->details->file)) {
-		new_icon = nautilus_icon_factory_get_icon_for_file (bookmark->details->file);
+	if (nautilus_file_check_if_ready (bookmark->details->file,
+					  NAUTILUS_FILE_ATTRIBUTES_FOR_ICON)) {
+		new_icon = nautilus_file_get_gicon (bookmark->details->file, 0);
 		if (nautilus_bookmark_icon_is_different (bookmark, new_icon)) {
-			g_free (bookmark->details->icon);
+			if (bookmark->details->icon) {
+				g_object_unref (bookmark->details->icon);
+			}
 			bookmark->details->icon = new_icon;
 			return TRUE;
 		}
-		g_free (new_icon);
+		g_object_unref (new_icon);
 	}
 
 	return FALSE;
@@ -319,35 +369,37 @@ nautilus_bookmark_update_icon (NautilusBookmark *bookmark)
 static void
 bookmark_file_changed_callback (NautilusFile *file, NautilusBookmark *bookmark)
 {
-	char *file_uri;
+	GFile *location;
 	gboolean should_emit_appearance_changed_signal;
 	gboolean should_emit_contents_changed_signal;
-
+	char *display_name;
+		
 	g_assert (NAUTILUS_IS_FILE (file));
 	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
 	g_assert (file == bookmark->details->file);
 
 	should_emit_appearance_changed_signal = FALSE;
 	should_emit_contents_changed_signal = FALSE;
-	file_uri = nautilus_file_get_uri (file);
+	location = nautilus_file_get_location (file);
 
-	if (!eel_uris_match (bookmark->details->uri, file_uri)) {
-		g_free (bookmark->details->uri);
-		bookmark->details->uri = file_uri;
+	if (!g_file_equal (bookmark->details->location, location) &&
+	    !nautilus_file_is_in_trash (file)) {
+		g_object_unref (bookmark->details->location);
+		bookmark->details->location = location;
 		should_emit_contents_changed_signal = TRUE;
 	} else {
-		g_free (file_uri);
+		g_object_unref (location);
 	}
 
-	if (nautilus_file_is_gone (file)) {
-		/* The file we were monitoring has been deleted,
+	if (nautilus_file_is_gone (file) ||
+	    nautilus_file_is_in_trash (file)) {
+		/* The file we were monitoring has been trashed, deleted,
 		 * or moved in a way that we didn't notice. Make 
 		 * a spanking new NautilusFile object for this 
 		 * location so if a new file appears in this place 
 		 * we will notice.
 		 */
 		nautilus_bookmark_disconnect_file (bookmark);
-		nautilus_bookmark_connect_file (bookmark);
 		should_emit_appearance_changed_signal = TRUE;		
 	} else if (nautilus_bookmark_update_icon (bookmark)) {
 		/* File hasn't gone away, but it has changed
@@ -356,6 +408,18 @@ bookmark_file_changed_callback (NautilusFile *file, NautilusBookmark *bookmark)
 		should_emit_appearance_changed_signal = TRUE;
 	}
 
+	if (!bookmark->details->has_custom_name) {
+		display_name = nautilus_file_get_display_name (file);
+
+		if (strcmp (bookmark->details->name, display_name) != 0) {
+			g_free (bookmark->details->name);
+			bookmark->details->name = display_name;
+			should_emit_appearance_changed_signal = TRUE;
+		} else {
+			g_free (display_name);
+		}
+	}
+
 	if (should_emit_appearance_changed_signal) {
 		g_signal_emit (bookmark, signals[APPEARANCE_CHANGED], 0);
 	}
@@ -377,7 +441,9 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
 	const char *icon_name;
 
 
-	g_free (bookmark->details->icon);
+	if (bookmark->details->icon) {
+		g_object_unref (bookmark->details->icon);
+	}
 
 	if (nautilus_bookmark_uri_known_not_to_exist (bookmark)) {
 		icon_name = MISSING_BOOKMARK_ICON_NAME;
@@ -385,7 +451,7 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
 		icon_name = GENERIC_BOOKMARK_ICON_NAME;
 	}
 	
-	bookmark->details->icon = g_strdup (icon_name);
+	bookmark->details->icon = g_themed_icon_new (icon_name);
 }
 
 /**
@@ -403,9 +469,9 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
  * 
  **/
 NautilusBookmark *
-nautilus_bookmark_new (const char *uri, const char *name)
+nautilus_bookmark_new (GFile *location, const char *name)
 {
-	return nautilus_bookmark_new_with_icon (uri, name, NULL);
+	return nautilus_bookmark_new_with_icon (location, name, TRUE, NULL);
 }
 
 static void
@@ -422,7 +488,7 @@ nautilus_bookmark_disconnect_file (NautilusBookmark *bookmark)
 	}
 
 	if (bookmark->details->icon != NULL) {
-		g_free (bookmark->details->icon);
+		g_object_unref (bookmark->details->icon);
 		bookmark->details->icon = NULL;
 	}
 }
@@ -430,6 +496,8 @@ nautilus_bookmark_disconnect_file (NautilusBookmark *bookmark)
 static void
 nautilus_bookmark_connect_file (NautilusBookmark *bookmark)
 {
+	char *display_name;
+	
 	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
 
 	if (bookmark->details->file != NULL) {
@@ -437,7 +505,7 @@ nautilus_bookmark_connect_file (NautilusBookmark *bookmark)
 	}
 
 	if (!nautilus_bookmark_uri_known_not_to_exist (bookmark)) {
-		bookmark->details->file = nautilus_file_get (bookmark->details->uri);
+		bookmark->details->file = nautilus_file_get (bookmark->details->location);
 		g_assert (!nautilus_file_is_gone (bookmark->details->file));
 
 		g_signal_connect_object (bookmark->details->file, "changed",
@@ -452,22 +520,35 @@ nautilus_bookmark_connect_file (NautilusBookmark *bookmark)
 			nautilus_bookmark_set_icon_to_default (bookmark);
 		}
 	}
+	
+	if (!bookmark->details->has_custom_name &&
+	    bookmark->details->file && 
+	    nautilus_file_check_if_ready (bookmark->details->file, NAUTILUS_FILE_ATTRIBUTE_INFO)) {
+		    display_name = nautilus_file_get_display_name (bookmark->details->file);
+		    if (strcmp (bookmark->details->name, display_name) != 0) {
+			    g_free (bookmark->details->name);
+			    bookmark->details->name = display_name;
+		    } else {
+			    g_free (display_name);
+		    }
+	}
 }
 
 NautilusBookmark *
-nautilus_bookmark_new_with_icon (const char *uri, const char *name, 
-				 const char *icon)
+nautilus_bookmark_new_with_icon (GFile *location, const char *name, gboolean has_custom_name,
+				 GIcon *icon)
 {
 	NautilusBookmark *new_bookmark;
 
 	new_bookmark = NAUTILUS_BOOKMARK (g_object_new (NAUTILUS_TYPE_BOOKMARK, NULL));
-	g_object_ref (new_bookmark);
-	gtk_object_sink (GTK_OBJECT (new_bookmark));
+	g_object_ref_sink (new_bookmark);
 
 	new_bookmark->details->name = g_strdup (name);
-	new_bookmark->details->uri = g_strdup (uri);
-
-	new_bookmark->details->icon = g_strdup (icon);
+	new_bookmark->details->location = g_object_ref (location);
+	new_bookmark->details->has_custom_name = has_custom_name;
+	if (icon) {
+		new_bookmark->details->icon = g_object_ref (icon);
+	}
 
 	nautilus_bookmark_connect_file (new_bookmark);
 
@@ -480,7 +561,7 @@ create_image_widget_for_bookmark (NautilusBookmark *bookmark)
 	GdkPixbuf *pixbuf;
 	GtkWidget *widget;
 
-	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, NAUTILUS_ICON_SIZE_FOR_MENUS, FALSE);
+	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, GTK_ICON_SIZE_MENU);
 	if (pixbuf == NULL) {
 		return NULL;
 	}
@@ -503,10 +584,13 @@ nautilus_bookmark_menu_item_new (NautilusBookmark *bookmark)
 {
 	GtkWidget *menu_item;
 	GtkWidget *image_widget;
-	GtkWidget *label;
-	char *display_name;
+	GtkLabel *label;
 	
-	menu_item = gtk_image_menu_item_new ();
+	menu_item = gtk_image_menu_item_new_with_label (bookmark->details->name);
+	label = GTK_LABEL (GTK_BIN (menu_item)->child);
+	gtk_label_set_use_underline (label, FALSE);
+	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (label, ELLIPSISED_MENU_ITEM_MIN_CHARS);
 
 	image_widget = create_image_widget_for_bookmark (bookmark);
 	if (image_widget != NULL) {
@@ -514,13 +598,6 @@ nautilus_bookmark_menu_item_new (NautilusBookmark *bookmark)
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
 					       image_widget);
 	}
-	display_name = eel_truncate_text_for_menu_item (bookmark->details->name);
-	label = gtk_label_new (display_name);
-	g_free (display_name);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-
-	gtk_container_add (GTK_CONTAINER (menu_item), label);
-	gtk_widget_show (label);
 
 	return menu_item;
 }
@@ -532,10 +609,10 @@ nautilus_bookmark_uri_known_not_to_exist (NautilusBookmark *bookmark)
 	gboolean exists;
 
 	/* Convert to a path, returning FALSE if not local. */
-	path_name = gnome_vfs_get_local_path_from_uri (bookmark->details->uri);
-	if (path_name == NULL) {
+	if (!g_file_is_native (bookmark->details->location)) {
 		return FALSE;
 	}
+	path_name = g_file_get_path (bookmark->details->location);
 
 	/* Now check if the file exists (sync. call OK because it is local). */
 	exists = g_file_test (path_name, G_FILE_TEST_EXISTS);
@@ -543,3 +620,17 @@ nautilus_bookmark_uri_known_not_to_exist (NautilusBookmark *bookmark)
 
 	return !exists;
 }
+
+void
+nautilus_bookmark_set_scroll_pos (NautilusBookmark      *bookmark,
+				  const char            *uri)
+{
+	g_free (bookmark->details->scroll_file);
+	bookmark->details->scroll_file = g_strdup (uri);
+}
+
+char *
+nautilus_bookmark_get_scroll_pos (NautilusBookmark      *bookmark)
+{
+	return g_strdup (bookmark->details->scroll_file);
+}
